
//
// Sky.java -- an applet which draws a star chart.
//	Copyright 1996 by Akkana Peck.
// You may use or distribute this code under the terms of the GPL.
//

package Akk.Astro;

import java.awt.*;
import java.applet.*;
import java.util.Date;

public class Sky extends Applet
{
    final static String WHITEonBLACK = "Black";
    final static String BLACKonWHITE = "White";

    // Copies of the UI components we maintain:
    StarChart starchart;
    TextField dateField, raField, decField, decspanField, numDaysField;
    TextField magField;
    Label mesgField;
    Choice invertMenu;
    Button printButton;
    Panel controlPanel;

    public String getAppletInfo() {
	return "Star Chart / Planet Locator.\nCopyright 1996 by Akkana Peck";
    }

    public String[][] getParameterInfo() {
	String[][] info = {
	    { "RA", "double", "Right Ascension of chart center" },
	    { "Dec", "double", "Declination of chart center" },
	    { "decspan", "double", "Declination span in degrees" },
	    { "time", "double",
	      "Time of chart (optional, defaults to current)" },
	    { "numDays", "double",
	      "Calc. orbits over this many days (optional)" },
	    { "magnitude", "double",
	      "Star min. magnitude (controls time to load database)" },
	    { "StarURL", "String", "URL from which to get the star chart" },
	};
	return info;
    }

    public void init()
    {
	String s;

	String StarURLname = getParameter("StarURL");
	// What's the default for this?

	double RAcenter, Deccenter;
	s = getParameter("RA");
	if (s != null)
	    RAcenter = Double.valueOf(s).doubleValue();
	else
	    RAcenter = 19;

    	s = getParameter("Dec");
	if (s != null)
	    Deccenter = Double.valueOf(s).doubleValue();
	else
	    Deccenter = 20;

	double decspan;
    	s = getParameter("decspan");
	if (s != null)
	    decspan = Double.valueOf(s).doubleValue();
	else
	    decspan = 40;

	int numDays;
    	s = getParameter("numDays");
	if (s != null)
	    numDays = Integer.valueOf(s).intValue();
	else
	    numDays = 1;

	int lmag;
    	s = getParameter("magnitude");
	if (s != null)
	    lmag = Integer.valueOf(s).intValue();
	else
	    lmag = 7;

	//
	// Now we have all the parameters.  Set some layout info:
	//
	setLayout(new BorderLayout(0, 1));

	//
	// Create the star chart object
	//
	starchart = new StarChart(RAcenter, Deccenter, decspan,
				  StarURLname, numDays, lmag);
	add("North", starchart);

	// Make a panel of controls at the bottom
	controlPanel = new Panel();
	GridBagLayout gridbag = new GridBagLayout();
	controlPanel.setLayout(gridbag);

	GridBagConstraints constraints = new GridBagConstraints();
	constraints.gridy = 0;
	constraints.fill = GridBagConstraints.BOTH;
	constraints.anchor = GridBagConstraints.NORTHWEST;
	constraints.insets = new Insets(2, 2, 2, 2);
	constraints.weightx = 1;
	constraints.weighty = 0;
	constraints.gridwidth = 1;

	constraints.gridx = 0;
	Label labl = new Label("Date:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	StarDate now = new StarDate();
	dateField = new TextField(now.toDateString(), 12);
	constraints.gridx = GridBagConstraints.RELATIVE;
	gridbag.setConstraints(dateField, constraints);
	controlPanel.add(dateField);

	labl = new Label("# Days:");
	labl.setAlignment(Label.RIGHT);
	controlPanel.add(labl);
	numDaysField = new TextField(new Integer((int)numDays).toString(), 5);
	gridbag.setConstraints(numDaysField, constraints);
	controlPanel.add(numDaysField);

	labl = new Label("Sky Color:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	invertMenu = new Choice();
	invertMenu.addItem(WHITEonBLACK);
	invertMenu.addItem(BLACKonWHITE);
	gridbag.setConstraints(invertMenu, constraints);
	controlPanel.add(invertMenu);

	// Start a new line:
	++constraints.gridy;
	constraints.gridx = 0;

	labl = new Label("R.A.:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	raField = new TextField(new Integer((int)RAcenter).toString(), 3);
	constraints.gridx = GridBagConstraints.RELATIVE;
	gridbag.setConstraints(raField, constraints);
	controlPanel.add(raField);

	labl = new Label("Dec.:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	decField = new TextField(new Integer((int)Deccenter).toString(), 3);
	gridbag.setConstraints(decField, constraints);
	controlPanel.add(decField);

	labl = new Label("Dec. span:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	decspanField = new TextField(new Integer((int)decspan).toString(), 3);
	gridbag.setConstraints(decspanField, constraints);
	controlPanel.add(decspanField);

	// Start a new line:
	++constraints.gridy;
	constraints.gridx = 0;

	mesgField = new Label("Initializing");
	constraints.weightx = 100;
	constraints.gridwidth = 4;
	gridbag.setConstraints(mesgField, constraints);
	controlPanel.add(mesgField);
	starchart.setMessages(mesgField);

	constraints.weightx = 1;
	constraints.gridx = GridBagConstraints.RELATIVE;
	constraints.gridwidth = 1;

	labl = new Label("Magnitude:");
	labl.setAlignment(Label.RIGHT);
	gridbag.setConstraints(labl, constraints);
	controlPanel.add(labl);

	magField = new TextField(new Integer((int)lmag).toString(), 3);
	constraints.gridwidth = GridBagConstraints.REMAINDER;
	gridbag.setConstraints(magField, constraints);
	controlPanel.add(magField);

	/* Print button disabled for now because we don't know how to print.
	printButton = new Button("Print");
	gridbag.setConstraints(printButton, constraints);
	controlPanel.add(printButton);
	*/

	add("South", controlPanel);
    }

    public void layout() {
	super.layout();
	if (starchart != null && controlPanel != null)
	    starchart.resize(size().width,
			     size().height - controlPanel.bounds().height);
    }

    public boolean handleEvent(Event event)
    {
	if (event.target == invertMenu)
	{
	    if (((String)(event.arg)).equals(WHITEonBLACK))
		starchart.invert(false);
	    else if (((String)(event.arg)).equals(BLACKonWHITE))
		starchart.invert(true);
	    starchart.repaint();

	} else if (event.target == printButton) {
	    printAll(getGraphics());

	} else if (event.id == Event.ACTION_EVENT &&
		   ( event.target == dateField ||
		     event.target == numDaysField ) )
	{
	    StarDate newdate;
	    int ndays;
	    try {
		newdate = new StarDate(dateField.getText());
		// Might throw IllegalArgumentException if user
		// typed in the wrong format ...
		ndays = new Integer(numDaysField.getText()).intValue();
		// Might throw NumberFormatException if user
		// typed in the wrong format ...

		// See comment under next clause ...
		numDaysField.nextFocus();

		starchart.setDate(newdate);
		starchart.setNumDays(ndays);
		starchart.repaint();
	    }
	    catch (NumberFormatException e) {
		numDaysField.setText(new Integer(starchart.getNumDays())
				     .toString());
	    }
	    catch (IllegalArgumentException e) {
		dateField.setText(starchart.getDate().toDateString());
		// should display errmsg ...
	    }
	} else if (event.id == Event.ACTION_EVENT &&
		   ( event.target == raField ||
		     event.target == decField ||
		     event.target == decspanField ||
		     event.target == magField ))
	{
	    try {
		Double new_ra = new Double(raField.getText());
		Double new_dec = new Double(decField.getText());
		Double new_decspan = new Double(decspanField.getText());
		Double new_mag = new Double(magField.getText());

		// For some reason, the component which was changed
		// grabs the focus and won't let anyone else have it.
		// Having any of the three components call nextFocus()
		// here seems to cure this problem, regardless of which
		// component calls it.
		magField.nextFocus();

		starchart.setStarMagnitude(new_mag.doubleValue());
		starchart.setCoordinates(new_ra.doubleValue(),
					 new_dec.doubleValue(),
					 new_decspan.doubleValue());
		starchart.repaint();
	    }
	    catch (IllegalArgumentException e) {
		raField.setText(new Double(starchart.getRA()).toString());
		decField.setText(new Double(starchart.getDec()).toString());
		decspanField.setText
		    (new Double(starchart.getDecSpan()).toString());
		// should display errmsg ...
	    }
	}
	return super.handleEvent(event);
    }
}

